# Generated by pip2nix {{ pip2nix_version }}
# Adjust to your needs, e.g. to provide C libraries.

{ pkgs, basePythonPackages }:

self: super: {

  # Example adjustment for lxml: It needs a few C libraries
  #
  # lxml = super.lxml.override (attrs: {
  #   buildInputs = with self; [
  #     pkgs.libxml2
  #     pkgs.libxslt
  #   ];
  # });

  # Common needs

  # setuptools - avoid that we end up in a recursion
  # inherit (basePythonPackages) setuptools;

}
